import click
import logging
from shell_database.store.store import DataStore
from shell_database.store.encryption import EncryptionManager


_data_store = None
_encryption_manager = None


def setup():
    global _data_store, _encryption_manager
    _data_store = DataStore()
    _encryption_manager = EncryptionManager()


@click.group()
@click.option('-v', '--verbose', help='Enable verbose output', default=False, is_flag=True)
@click.option('-d', '--debug', help='Enable debug output', default=False, is_flag=True)
def cli(verbose: bool = False, debug: bool = False):
    if verbose:
        logging.getLogger().setLevel(level='INFO')
    if debug:
        logging.getLogger().setLevel(level='DEBUG')
        logging.debug('debug output enabled')
    setup()


@cli.command(help="Add a new key value pair")
@click.argument('key')
@click.argument('value')
@click.option('-e', '--encrypt', help="Encrypt the value before storing it", is_flag=True, required=False)
def add(key, value, encrypt: bool):
    if encrypt:
        value = _encryption_manager.encrypt(value)
    _data_store.add(key, value)


@cli.command(help="Get the value from a key")
@click.argument('key')
@click.option('-d', '--decrypt', help="Decrypt the value stored", is_flag=True, required=False)
def get(key, decrypt):
    value = _data_store.get(key)
    if decrypt and isinstance(value, bytes):
        value = _encryption_manager.decrypt(value)
    if value:
        print(value)


@cli.command('list', help="List the stored keys")
@click.argument('pattern', required=False)
@click.option('-c', '--count', help="Counts the number of keys found", required=False, is_flag=True)
def list_keys(pattern: str = None, count: bool = False):
    keys = _data_store.list_keys(pattern)
    if count:
        print(len(keys))
    for key in keys:
        print(key)


if __name__ == '__main__':
    cli()



from pathlib import Path
from fuzzywuzzy import process
from sqlitedict import SqliteDict


class DataStore:

    DEFAULT_FILE = Path.home() / '.shdb/store.sqlite'

    def __init__(self, filename: Path = None):
        self.store = filename or DataStore.DEFAULT_FILE
        if not self.store.parent.exists():
            self.store.parent.mkdir(parents=True, exist_ok=True)
        sqlite = SqliteDict(self.store)
        sqlite.close()
        self.store.chmod(0o600)

    def add(self, key, value):
        with SqliteDict(str(self.store), autocommit=True) as db:
            db[key] = value

    def get(self, key) -> str:
        with SqliteDict(str(self.store), flag='r', autocommit=True) as db:
            try:
                return db[key]
            except KeyError:
                return None

    def list_keys(self, pattern: str = None):
        with SqliteDict(str(self.store), flag='r', autocommit=True) as db:
            keys = [k for k, _ in db.items()]
        if pattern:
            keys = process.extract(pattern, keys)
            return [k for k, rating in keys if rating > 50]
        return keys



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shell_database', 'shell_database.store']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'coveralls>=2.1.2,<3.0.0',
 'cryptography>=3.1,<4.0',
 'flake8>=3.8.3,<4.0.0',
 'fuzzywuzzy>=0.18.0,<0.19.0',
 'sqlitedict>=1.7.0,<2.0.0']

entry_points = \
{'console_scripts': ['shdb = shell_database.shdb:cli']}

setup_kwargs = {
    'name': 'shell-database',
    'version': '0.1.1',
    'description': 'A key value store for the shell',
    'long_description': '# Shell Database\n\n## Installation\n\nYou can install shell-database from `pypi.org`:\n```console\n> pip install shell_database\n```\n\n## Getting started\n\n### Adding a new key value pair\n```\n> shdb add name "John Doe"\n```\n\n### Adding a new key value pair with encryption\n\n```\n> shdb add password <your_password> --encrypt\n```\n\n### Getting the value of a key\n\n```console\n> shdb get name\nJohn Doe\n```\n\n### Decrypting and encrypted value\n\n```console\n> shdb get password\nb\'51b8684c4dc77da0979f1b647caa707c\'\n\n> shdb get password --decrypt\n<your_password>\n```\n\n### Integrating with other tools\n\n```console\n> shdb add az-rg azure-resource-group-123\n\n> az postgres db create --resource-group $(shdb get az-rg) --server-name server_name --name database\n```\n\n## License\n\nMIT License\n\n## Disclaimer\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.',
    'author': 'José Coelho',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jcoelho93/shell-database',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path
from shell_database import __encoding__
from cryptography.fernet import Fernet


class EncryptionManager:

    DEFAULT_KEY_FILE = Path.home() / '.shdb/secret.key'

    def __init__(self, filename: Path = None):
        filename = filename or EncryptionManager.DEFAULT_KEY_FILE
        if not filename.exists():
            self.key = self.__generate_key(filename)
        else:
            self.key = self.__load_key(filename)

    def encrypt(self, value: str) -> bytes:
        encoded = value.encode(__encoding__)
        f = Fernet(self.key)
        encrypted = f.encrypt(encoded)
        return encrypted

    def decrypt(self, data: bytes) -> str:
        f = Fernet(self.key)
        decrypted = f.decrypt(data)
        return decrypted.decode(__encoding__)

    def __generate_key(self, filename: Path):
        filename.parent.mkdir(parents=True, exist_ok=True)
        key = Fernet.generate_key()
        with filename.open(mode='wb') as key_file:
            filename.chmod(0o600)
            key_file.write(key)
        return key

    def __load_key(self, filename: Path):
        return filename.open(mode='r').read()
